@@ -162,8 +162,7 @@ std::string StorageDevice::fetch_basic_data_and_parse(hz::intrusive_ptr<CmdexSyn
 
 	// We don't use "--all" - it may cause really screwed up the output (tests, etc...).
 	// This looks just like "--info" only on non-smart devices.
-	// --info --health --capabilities
-	std::string error_msg = execute_device_smartctl("-i -H -c", smartctl_ex, this->info_output_, true);  // set type to invalid if needed
+	std::string error_msg = execute_device_smartctl("--info --health --capabilities", smartctl_ex, this->info_output_, true);  // set type to invalid if needed
 
 	// Smartctl 5.39 cvs/svn version defaults to usb type on at least linux and windows.
 	// This means that the old SCSI identify command isn't executed by default,
@@ -320,15 +319,15 @@ std::string StorageDevice::fetch_data_and_parse(hz::intrusive_ptr<CmdexSync> sma
 	std::string output;
 	std::string error_msg;
 
-	// instead of -a, we use all the individual options -a encompasses, so that
-	// an addition to default -a output won't affect us.
+	// instead of -x, we use all the individual options -x encompasses, so that
+	// an addition to default -x output won't affect us.
 	if (this->get_type_argument() == "scsi") {  // not sure about correctness... FIXME probably fails with RAID/scsi
 		// This doesn't do much yet, but just in case...
-		// SCSI equivalent of -a:
-		error_msg = execute_device_smartctl("-H -i -A -l error -l selftest", smartctl_ex, output);
+		// SCSI equivalent of -x:
+		error_msg = execute_device_smartctl("--health --info --attributes --log=error --log=selftest --log=background --log=sasphy", smartctl_ex, output);
 	} else {
-		// ATA equivalent of -a:
-		error_msg = execute_device_smartctl("-H -i -c -A -l error -l selftest -l selective",
+		// ATA equivalent of -x:
+		error_msg = execute_device_smartctl("--health --info --get=all --capabilities --attributes --format=brief --log=xerror,50,error --log=xselftest,50,selftest --log=selective --log=directory --log=scttemp --log=scterc --log=devstat --log=sataphy",
 				smartctl_ex, output, true);  // set type to invalid if needed
 	}
 	// See notes above (in fetch_basic_data_and_parse()).
@@ -423,8 +422,7 @@ A mandatory SMART command failed: exiting. To continue, add one or more '-T perm
 */
 
 	std::string output;
-	// --smart=on --saveauto=on, --smart=off
-	std::string error_msg = execute_device_smartctl((b ? "-s on -S on" : "-s off"), smartctl_ex, output);
+	std::string error_msg = execute_device_smartctl((b ? "--smart=on --saveauto=on" : "--smart=off"), smartctl_ex, output);
 	if (!error_msg.empty())
 		return error_msg;
 
@@ -458,8 +456,7 @@ SMART Automatic Offline Testing Disabled.
 A mandatory SMART command failed: exiting. To continue, add one or more '-T permissive' options.
 */
 	std::string output;
-	// --offlineauto=on, --offlineauto=off
-	std::string error_msg = execute_device_smartctl((b ? "-o on" : "-o off"), smartctl_ex, output);
+	std::string error_msg = execute_device_smartctl((b ? "--offlineauto=on" : "--offlineauto=off"), smartctl_ex, output);
 	if (!error_msg.empty())
 		return error_msg;
 
@@ -654,6 +651,31 @@ std::string StorageDevice::get_extra_arguments() const
 
 
 
+void StorageDevice::set_drive_letters(const std::vector< char >& letters)
+{
+	drive_letters_ = letters;
+}
+
+
+
+const std::vector< char >& StorageDevice::get_drive_letters() const
+{
+	return drive_letters_;
+}
+
+
+
+std::string StorageDevice::format_drive_letters() const
+{
+	std::vector<std::string> drive_letters_decorated;
+	for (std::size_t i = 0; i < drive_letters_.size(); ++i) {
+		drive_letters_decorated.push_back(std::string() + (char)std::toupper(drive_letters_[i]) + ":");
+	}
+	return hz::string_join(drive_letters_decorated, ", ");
+}
+
+
+
 bool StorageDevice::get_is_virtual() const
 {
 	return is_virtual_;
