@@ -124,8 +124,12 @@ class StorageAttribute {
 		{ }
 
 
+		/// Format raw value with commas (if it's a number)
+		std::string format_raw_value() const;
+
+
 		int32_t id;  ///< Attribute ID (most vendors agree on this)
-		std::string flag;  ///< Some have it in 0xXXXX format, others in "PO--C-" format (some WD drives?).
+		std::string flag;  ///< "Old" format is "0xXXXX", "brief" format is "PO--C-".
 		hz::OptionalValue<uint8_t> value;  ///< Normalized value. May be unset ("---").
 		hz::OptionalValue<uint8_t> worst;  ///< Worst ever value. May be unset ("---").
 		hz::OptionalValue<uint8_t> threshold;  ///< Threshold for normalized value. May be unset ("---").
@@ -144,6 +148,35 @@ std::ostream& operator<< (std::ostream& os, const StorageAttribute& p);
 
 
 
+/// Holds one line of "devstat" subsection
+class StorageStatistic {
+	public:
+
+		/// Constructor
+		StorageStatistic() : is_header(false), value_int(0), page(0), offset(0)
+		{ }
+
+		/// Whether the normalization flag is present
+		bool is_normalized() const
+		{
+			return flags.find('N') != flags.npos;
+		}
+
+		bool is_header;  ///< If the line is a header
+		std::string flags;  ///< Flags in "NDC" / "---" format
+		std::string value;  ///< Value as a string, as presented by smartctl (formatted).
+		int64_t value_int;  ///< Same as value, but parsed as int64.
+		int64_t page;  ///< Page
+		int64_t offset;  ///< Offset in page
+};
+
+
+/// Output operator for debug purposes
+std::ostream& operator<< (std::ostream& os, const StorageStatistic& p);
+
+
+
+
 /// Holds one error block of "error log" subsection
 class StorageErrorBlock {
 	public:
@@ -155,6 +188,12 @@ class StorageErrorBlock {
 		/// Get readable error types from reported types
 		static std::string get_readable_error_types(const std::vector<std::string>& types);
 
+		/// Get warning level (warning_t) for an error type
+		static int get_warning_level_for_error_type(std::string& type);
+
+		/// Format lifetime hours with comma
+		std::string format_lifetime_hours() const;
+
 		uint32_t error_num;  ///< Error number
 		uint32_t lifetime_hours;  ///< When the error occurred (in lifetime hours)
 		std::string device_state;  ///< Device state during the error - "active or idle", standby, etc...
@@ -249,13 +288,17 @@ class StorageSelftestEntry {
 		}
 
 
+		/// Format lifetime hours with comma
+		std::string format_lifetime_hours() const;
+
+
 		uint32_t test_num;  ///< Test number. always starts from 1. larger means older or newer, depending on model. 0 for capability.
 		std::string type;  ///< Extended offline, Short offline, Conveyance offline, etc... . capability: unused.
 		std::string status_str;  ///< Self-test routine in progress, Completed without error, etc... (as reported by log or capability)
 		status_t status;  ///< same as status_str, but from enum
 		int8_t remaining_percent;  ///< Remaining %. 0% for completed, 90% for started. -1 if n/a.
 		uint32_t lifetime_hours;  ///< When the test happened (in lifetime hours). capability: unused.
-		std::string lba_of_first_error;  ///< LBA of the first error. "-" or value (format?). capability: unused.
+		std::string lba_of_first_error;  ///< LBA of the first error. "-" or value (format? usually hex). capability: unused.
 };
 
 
@@ -279,6 +322,7 @@ class StorageProperty {
 
 			value_type_capability,  ///< For "capabilities" subsection (non-time-interval blocks only)
 			value_type_attribute,  ///< For "attributes" subsection
+			value_type_statistic,  ///< For "devstat" subsection
 			value_type_error_block,  // For "error_log" subsection
 			value_type_selftest_entry  // For "selftest_log" subsection
 		};
@@ -294,6 +338,7 @@ class StorageProperty {
 				case value_type_time_length: return "time_length";
 				case value_type_capability: return "capability";
 				case value_type_attribute: return "attribute";
+				case value_type_statistic: return "statistic";
 				case value_type_error_block: return "error_block";
 				case value_type_selftest_entry: return "selftest_entry";
 			}
@@ -328,9 +373,14 @@ class StorageProperty {
 			subsection_health,  ///< Overall-health (-H, --health)
 			subsection_capabilities,  ///< General SMART Values, aka Capabilities (-c, --capabilities)
 			subsection_attributes,  ///< Attributes (-A, --attributes). These need decoding.
-			subsection_error_log,  ///< Error Log (-l error)
-			subsection_selftest_log,  ///< Self-test log (-l selftest)
-			subsection_selective_selftest_log  ///< Selective self-test log and settings
+			subsection_devstat,  ///< Device statistics (--log=devstat). These need decoding.
+			subsection_error_log,  ///< Error Log (--log=error)
+			subsection_selftest_log,  ///< Self-test log (--log=selftest)
+			subsection_selective_selftest_log,  ///< Selective self-test log (--log=selective)
+			subsection_temperature_log,  ///< SCT temperature (current and history) (--log=scttemp)
+			subsection_erc_log,  ///< SCT Error Recovery Control settings (--log=scterc)
+			subsection_phy_log,  ///< Phy log (--log=sataphy)
+			subsection_directory_log,  ///< Directory log (--log=directory)
 		};
 
 		/// Get displayable subsection type name
@@ -341,9 +391,14 @@ class StorageProperty {
 				case subsection_health: return "health";
 				case subsection_capabilities: return "capabilities";
 				case subsection_attributes: return "attributes";
+				case subsection_devstat: return "devstat";
 				case subsection_error_log: return "error_log";
 				case subsection_selftest_log: return "selftest_log";
 				case subsection_selective_selftest_log: return "selective_selftest_log";
+				case subsection_temperature_log: return "temperature_log";
+				case subsection_erc_log: return "erc_log";
+				case subsection_phy_log: return "phy_log";
+				case subsection_directory_log: return "directory_log";
 			}
 			return "[error]";
 		}
@@ -408,7 +463,6 @@ class StorageProperty {
 		}
 
 
-
 		std::string reported_name;  ///< Property name as reported by smartctl.
 		std::string generic_name;  ///< Generic (internal) name. May be same as reported_name, or something more program-identifiable.
 		std::string readable_name;  ///< Readable property name. May be same as reported_name, or something more user-readable. Possibly translatable.
@@ -436,6 +490,7 @@ class StorageProperty {
 
 		StorageCapability value_capability;  ///< Value (if it's a capability)
 		StorageAttribute value_attribute;  ///< Value (if it's an attribute)
+		StorageStatistic value_statistic;  ///< Value (if it's a statistic from devstat)
 		StorageErrorBlock value_error_block;  ///< Value (if it's a error block)
 		StorageSelftestEntry value_selftest_entry;  ///< Value (if it's a self-test entry)
 
