@@ -13,6 +13,7 @@
 #define GSC_INFO_WINDOW_H
 
 #include <gtkmm.h>
+#include <map>
 
 #include "applib/app_ui_res_utils.h"
 #include "applib/storage_device.h"
@@ -34,8 +35,10 @@ class GscInfoWindow : public AppUIResWidget<GscInfoWindow, true> {
 		GscInfoWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& ref_ui);
 
 		/// Virtual destructor
-		virtual ~GscInfoWindow()
-		{ }
+		virtual ~GscInfoWindow();
+
+		// Overridden from InstanceManager
+		virtual void obj_destroy();
 
 
 		/// Set the drive to show
@@ -97,9 +100,15 @@ class GscInfoWindow : public AppUIResWidget<GscInfoWindow, true> {
 		void on_test_stop_button_clicked();
 
 
-		// Callback attached to StorageDevice change signal.
+		/// Callback attached to StorageDevice change signal.
 		void on_drive_changed(StorageDevice* pdrive);
 
+		/// Callback
+		bool on_treeview_button_press_event(GdkEventButton* button_event, Gtk::Menu* menu, Gtk::TreeView* treeview);
+
+		/// Callback
+		void on_treeview_menu_copy_clicked(Gtk::TreeView* treeview);
+
 
 	private:
 
@@ -114,13 +123,22 @@ class GscInfoWindow : public AppUIResWidget<GscInfoWindow, true> {
 
 		// --------- Data members
 
+		std::map<std::string, Gtk::Menu*> treeview_menus;  ///< Context menus
+
 		// tab headers, to perform their coloration
 		Glib::ustring tab_identity_name;  ///< Tab header name
-		Glib::ustring tab_capabilities_name;  ///< Tab header name
 		Glib::ustring tab_attributes_name;  ///< Tab header name
-		Glib::ustring tab_error_log_name;  ///< Tab header name
-		Glib::ustring tab_selftest_log_name;  ///< Tab header name
+		Glib::ustring tab_statistics_name;  ///< Tab header name
 		Glib::ustring tab_test_name;  ///< Tab header name
+		Glib::ustring tab_error_log_name;  ///< Tab header name
+		Glib::ustring tab_temperature_name;  ///< Tab header name
+		Glib::ustring tab_advanced_name;  ///< Tab header name
+
+		Glib::ustring tab_capabilities_name;  ///< Tab header name
+		Glib::ustring tab_erc_name;  ///< Tab header name
+		Glib::ustring tab_selective_selftest_name;  ///< Tab header name
+		Glib::ustring tab_phy_name;  ///< Tab header name
+		Glib::ustring tab_directory_name;  ///< Tab header name
 
 		Gtk::Label* device_name_label;  ///< Top label
 
