@@ -56,9 +56,29 @@ GscMainWindow::GscMainWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& re
 	// iconview, gtkuimanager stuff (menus), custom labels
 	create_widgets();
 
+	// Size
+	{
+		int def_size_w = 0, def_size_h = 0;
+		rconfig::get_data("gui/main_window/default_size_w", def_size_w);
+		rconfig::get_data("gui/main_window/default_size_h", def_size_h);
+		if (def_size_w > 0 && def_size_h > 0) {
+			set_default_size(def_size_w, def_size_h);
+		}
+	}
+
 	// show the window first, scan later
 	show();
 
+	// Position (after the window has been shown)
+	{
+		int pos_x = 0, pos_y = 0;
+		rconfig::get_data("gui/main_window/default_pos_x", pos_x);
+		rconfig::get_data("gui/main_window/default_pos_y", pos_y);
+		if (pos_x > 0 && pos_y > 0) {  // to avoid situations where positions are not supported
+			this->move(pos_x, pos_y);
+		}
+	}
+
 	while (Gtk::Main::events_pending())  // allow the window to show
 		Gtk::Main::iteration();
 
@@ -109,6 +129,18 @@ GscMainWindow::GscMainWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& re
 			break;
 		}
 
+		{
+			// We require this version at runtime to support --get=all.
+			const double minimum_req_version = 5.43;
+			double version_double = 0;
+			if (hz::string_is_numeric<double>(version, version_double, false)) {
+				if (version_double < minimum_req_version) {
+					error_msg = "Smartctl version " + version + " found, " + hz::number_to_string(minimum_req_version) + " required.";
+					break;
+				}
+			}
+		}
+
 	} while (false);
 
 	bool smartctl_valid = error_msg.empty();
@@ -230,6 +262,7 @@ bool GscMainWindow::create_widgets()
 
 	"	<menu action='options_menu'>"
 	"		<menuitem action='" APP_ACTION_NAME(action_executor_log) "' />"
+	"		<menuitem action='" APP_ACTION_NAME(action_update_drivedb) "' />"
 	"		<menuitem action='" APP_ACTION_NAME(action_preferences) "' />"
 	"	</menu>"
 
@@ -337,6 +370,10 @@ bool GscMainWindow::create_widgets()
 		actiongroup_main->add((action_map[action_executor_log] = action),
 				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_executor_log));
 
+		action = Gtk::Action::create(APP_ACTION_NAME(action_update_drivedb), "Update Drive Database");
+		actiongroup_main->add((action_map[action_update_drivedb] = action),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_update_drivedb));
+
 		action = Gtk::Action::create(APP_ACTION_NAME(action_preferences), Gtk::Stock::PREFERENCES);
 		actiongroup_main->add((action_map[action_preferences] = action), Gtk::AccelKey("<alt>P"),
 				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_preferences));
@@ -502,13 +539,7 @@ void GscMainWindow::on_action_activated(GscMainWindow::action_t action_type)
 
 	switch (action_type) {
 		case action_quit:
-			// if at least one drive is having a test performed, disallow.
-			if (this->testing_active()) {
-				if (!ask_about_quit_on_test(*this)) {
-					break;
-				}
-			}
-			app_quit();  // ends the main loop
+			quit_requested();
 			break;
 
 		case action_view_details:
@@ -576,6 +607,12 @@ void GscMainWindow::on_action_activated(GscMainWindow::action_t action_type)
 			break;
 		}
 
+		case action_update_drivedb:
+		{
+			run_update_drivedb();
+			break;
+		}
+
 		case action_preferences:
 		{
 			GscPreferencesWindow* win = GscPreferencesWindow::create();  // destroyed on close
@@ -950,16 +987,19 @@ void GscMainWindow::update_status_widgets()
 		return;
 	}
 
-	std::string device = (drive->get_is_virtual() ? ("Virtual: " + drive->get_virtual_filename()) : drive->get_device_with_type());
-	std::string size = drive->get_device_size_str();
-	std::string model = (drive->get_model_name().empty() ? std::string("Unknown model") : drive->get_model_name());
-	std::string family = (drive->get_family_name().empty() ? "Unknown" : drive->get_family_name());
-	std::string family_fallback = (drive->get_family_name().empty() ? model : drive->get_family_name());
+	std::string device = Glib::Markup::escape_text(drive->get_is_virtual() ? ("Virtual: " + drive->get_virtual_filename()) : drive->get_device_with_type());
+	std::string size = Glib::Markup::escape_text(drive->get_device_size_str());
+	std::string model = Glib::Markup::escape_text(drive->get_model_name().empty() ? std::string("Unknown model") : drive->get_model_name());
+	std::string family = Glib::Markup::escape_text(drive->get_family_name().empty() ? "Unknown" : drive->get_family_name());
+	std::string family_fallback = Glib::Markup::escape_text(drive->get_family_name().empty() ? model : drive->get_family_name());
+	std::string drive_letters_str = Glib::Markup::escape_text(drive->format_drive_letters());
 
-
-	std::string info_str = device + (size.empty() ? "" : (", " + size)) + (model.empty() ? "" : (", " + model));
+	std::string info_str = device
+			+ (drive_letters_str.empty() ? "" : (" (<b>" + drive_letters_str + "</b>)"))
+			+ (size.empty() ? "" : (", " + size))
+			+ (model.empty() ? "" : (", " + model));
 	if (name_label) {
-		name_label->set_text(info_str);
+		name_label->set_markup(info_str);
 		app_gtkmm_set_widget_tooltip(*name_label, info_str, false);  // in case it doesn't fit
 	}
 
@@ -1096,6 +1136,35 @@ void GscMainWindow::rescan_devices()
 
 
 
+void GscMainWindow::run_update_drivedb()
+{
+	std::string smartctl_binary = get_smartctl_binary();
+
+	if (smartctl_binary.empty()) {
+		gui_show_error_dialog("Error Updating Drive Database", "Smartctl binary is not specified in configuration.", this);
+		return;
+	}
+
+	std::string update_binary;
+	hz::FsPath path(smartctl_binary);
+	if (path.is_absolute()) {
+		update_binary = path.get_dirname() + "/";
+	}
+	update_binary += "update-smart-drivedb";
+	update_binary = Glib::shell_quote(update_binary);
+
+#ifndef _WIN32
+	update_binary = "xterm -hold -e " + update_binary;
+#endif
+
+	hz::scoped_ptr<GError> spawn_error(0, g_error_free);
+	if (!g_spawn_command_line_async(update_binary.c_str(), &spawn_error.get_ref())) {
+		gui_show_error_dialog("Error Updating Drive Database", spawn_error->message, this);
+	}
+}
+
+
+
 bool GscMainWindow::add_device(const std::string& file, const std::string& type_arg, const std::string& extra_args)
 {
 #ifndef _WIN32  	// win32 doesn't have device files, so skip the check
@@ -1277,12 +1346,28 @@ void GscMainWindow::show_add_device_chooser()
 void GscMainWindow::show_load_virtual_file_chooser()
 {
 	static std::string last_dir;
+	if (last_dir.empty()) {
+		rconfig::get_data("gui/drive_data_open_save_dir", last_dir);
+	}
 	int result = 0;
 
+	Glib::RefPtr<Gtk::FileFilter> specific_filter = Gtk::FileFilter::create();
+	specific_filter->set_name("Text Files");
+	specific_filter->add_pattern("*.txt");
+
+	Glib::RefPtr<Gtk::FileFilter> all_filter = Gtk::FileFilter::create();
+	all_filter->set_name("All Files");
+	all_filter->add_pattern("*");
+
 #if GTK_CHECK_VERSION(3, 20, 0)
 	hz::scoped_ptr<GtkFileChooserNative> dialog(gtk_file_chooser_native_new(
 			"Load Data From...", this->gobj(), GTK_FILE_CHOOSER_ACTION_OPEN, NULL, NULL), g_object_unref);
 
+	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog.get()), specific_filter->gobj());
+	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog.get()), all_filter->gobj());
+
+	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog.get()), true);
+
 	if (!last_dir.empty()) {
 		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog.get()), last_dir.c_str());
 	}
@@ -1297,6 +1382,11 @@ void GscMainWindow::show_load_virtual_file_chooser()
 	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
 	dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
 
+	dialog.add_filter(specific_filter);
+	dialog.add_filter(all_filter);
+
+	dialog.set_select_multiple(true);
+
 	if (!last_dir.empty())
 		dialog.set_current_folder(last_dir);
 
@@ -1308,16 +1398,27 @@ void GscMainWindow::show_load_virtual_file_chooser()
 	switch (result) {
 		case Gtk::RESPONSE_ACCEPT:
 		{
-			std::string file;
+			std::vector<std::string> files;
 
 #if GTK_CHECK_VERSION(3, 20, 0)
-			file = app_ustring_from_gchar(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog.get())));
-			last_dir = hz::path_get_dirname(file);
+			hz::scoped_ptr<GSList> file_slist(gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dialog.get())), g_slist_free);
+			GSList* iterator = file_slist.get();
+			while(iterator) {
+				files.push_back(app_ustring_from_gchar((gchar*)iterator->data));
+				iterator = g_slist_next(iterator);
+			}
 #else
-			file = dialog.get_filename();  // in fs encoding
-			last_dir = dialog.get_current_folder();  // save for the future
+			files = dialog.get_filenames();  // in fs encoding
 #endif
-			this->add_virtual_drive(file);
+			if (!files.empty()) {
+				last_dir = hz::path_get_dirname(files.front());
+			}
+			rconfig::set_data("gui/drive_data_open_save_dir", last_dir);
+			for (size_t i = 0; i < files.size(); ++i) {
+				if (hz::File(files[i]).is_file()) {  // file chooser returns selected directories as well, ignore them.
+					this->add_virtual_drive(files[i]);
+				}
+			}
 			break;
 		}
 
@@ -1333,8 +1434,7 @@ void GscMainWindow::show_load_virtual_file_chooser()
 
 
 
-// by default, delete_event calls hide().
-bool GscMainWindow::on_delete_event_before(GdkEventAny* e)
+bool GscMainWindow::quit_requested()
 {
 	// if at least one drive is having a test performed, disallow.
 	if (this->testing_active()) {
@@ -1342,12 +1442,35 @@ bool GscMainWindow::on_delete_event_before(GdkEventAny* e)
 			return true;  // handled
 		}
 	}
+
+	// window size / pos
+	{
+		int window_w = 0, window_h = 0;
+		get_size(window_w, window_h);
+		rconfig::set_data("gui/main_window/default_size_w", window_w);
+		rconfig::set_data("gui/main_window/default_size_h", window_h);
+
+		int pos_x = 0, pos_y = 0;
+		get_position(pos_x, pos_y);
+		rconfig::set_data("gui/main_window/default_pos_x", pos_x);
+		rconfig::set_data("gui/main_window/default_pos_y", pos_y);
+	}
+
 	app_quit();  // ends the main loop
+
 	return true;  // event handled, don't call default virtual handler
 }
 
 
 
+// by default, delete_event calls hide().
+bool GscMainWindow::on_delete_event_before(GdkEventAny* e)
+{
+	return quit_requested();
+}
+
+
+
 
 
 
